//----------------------------------------------------------------------------
//  Header: CHistoCollection.h, v 1.0                         2008/11/19    //
//                                                                          //
//  Goofie ALICE Program Library                                            //
//  Copyright (C) ALICE collaboration, 2008                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#include <iostream>
#include <cstdlib>

#include "TObject.h"
#include "TFile.h"
#include "TList.h"
#include "TH1.h"
#include "TH2.h"

#include "CHistoCollection.h"

using namespace std;

CHistoCollection::CHistoCollection(TFile *_fp)
{
   fout        = _fp;
   fHistoList   = new TList();
   // fHistoList->SetOwner(true);  // necessary to delete object in the list in destructor -- error in ROOT
   
   CreateClientHistograms();
}

CHistoCollection::~CHistoCollection()
{
   cout << "calling CHistoCollection::~CHistoCollection() " << flush << endl;


   cout << "Info from <CHistoCollection::~CHistoCollection()>: Deleting list " << flush << endl;
   // delete List
   if(fHistoList) {
      delete fHistoList;
      fHistoList = NULL;
   }
}

void CHistoCollection::SaveListToFile()
{
   if(!fout) {
      cerr << "Error in <DHist::SaveListToFile>: File does not exist" << flush << endl;
      exit(-1);
   }
   // GO TO Proper file
   fout->cd();
   
   TObjLink *lnk = fHistoList->FirstLink();
   while(lnk) {
      TObject *tmp = lnk->GetObject();
      tmp->Write();
      lnk = lnk->Next();
   }
}


void CHistoCollection::AddObjectToBeSaved(TObject *_save)
{
   if(fHistoList && _save)
      fHistoList->Add(_save);
}

void  CHistoCollection::CreateClientHistograms()
{
   // pickup part
   hAmplitudeVsPeakPos = new TH2F("hAmplitudeVsPeakPos", "amplitude vs peak pos.", 512, 0.0, 512.0, 1024, 0.0, 1024.0);
   fHistoList->Add(hAmplitudeVsPeakPos);

   hWidthVsPeakPos = new TH2F("hWidthVsPeakPos","width vs peak pos.", 512, 0.0, 512.0, 100, 0.0, 200.0);
   fHistoList->Add(hWidthVsPeakPos);

   hOffsetVsPeakPos = new TH2F("hOffsetVsPeakPos","offset vs peak pos.", 512, 0.0, 512.0, 100, 0.0, 200.0);
   fHistoList->Add(hOffsetVsPeakPos);
   
   hChi2VsPeakPos = new TH2F("hChi2VsPeakPos","chi^{2} vs peak pos.", 512, 0.0, 512.0, 250, 0.0, 250.0);
   fHistoList->Add(hChi2VsPeakPos);
   
   hAreaVsPeakPos = new TH2F("hAreaVsPeakPos","Area vs peak pos.", 512, 0.0, 512.0, 1500, 0.0, 15000.0);
   fHistoList->Add(hAreaVsPeakPos);

   hAmplitudeVsAreaPickup = new TH2F("hAmplitudeVsAreaPickup","Amplitude vs area from Pickup.", 1500, 0.0, 15000.0, 1024, 0.0, 1024.0);
   fHistoList->Add(hAmplitudeVsAreaPickup);
   
   hPOverTemp1VsPickupPeakPosition = new TH2F("hPOverTemp1VsPickupPeakPosition", "P/T1 vs pickup position", 512, 0.0, 512.0, 1000, 2.0, 4.0);
   fHistoList->Add(hPOverTemp1VsPickupPeakPosition);

   hPOverTemp2VsPickupPeakPosition = new TH2F("hPOverTemp2VsPickupPeakPosition", "P/T2 vs pickup position", 512, 0.0, 512.0, 1000, 2.0, 4.0);
   fHistoList->Add(hPOverTemp2VsPickupPeakPosition);



   // trigger near
   hNearAmplitudeVsPickupPeakPosition = new TH2F("hNearAmplitudeVsPickupPeakPosition","Amplitude Near vs peak position Pickup", 1024, 0.0, 1024,  512, 0.0, 512.0);
   fHistoList->Add(hNearAmplitudeVsPickupPeakPosition);
  
   hNearTotalAmplitudeVsPickupPeakPosition = new TH2F("hNearTotalAmplitudeVsPickupPeakPosition","(Amplitude+Offset) Near vs peak position Pickup", 1024, 0.0, 1024,  512, 0.0, 512.0);
   fHistoList->Add(hNearTotalAmplitudeVsPickupPeakPosition);

   hNearAreaVsPickupPeakPosition  = new TH2F("hNearAreaVsPickupPeakPosition","Area Near vs peak position Pickup",  512, 0.0, 512.0, 1500, 0.0, 15000.0);
   fHistoList->Add(hNearAreaVsPickupPeakPosition);
   
   hNearAreaVsPickupArea = new TH2F("hNearAreaVsPickupArea","Area Near vs area Pickup", 1500, 0.0, 15000.0,  1500, 0.0, 15000.0);
   fHistoList->Add(hNearAreaVsPickupArea);

   hNearPositionVsPickupPeakPosition = new TH2F("hNearPositionVsPickupPeakPosition","Near peak position vs Pickup peak position",  512, 0.0, 512.0,  512, 0.0, 512.0);
   fHistoList->Add(hNearPositionVsPickupPeakPosition);

   hNearWidthVsPickupPeakPosition = new TH2F("hNearWidthVsPickupPeakPosition","Near peak width vs Pickup peak position", 512, 0.0, 512.0, 100, 0.0, 100.0);
   fHistoList->Add(hNearWidthVsPickupPeakPosition);

   hNearAmplitudeVsNearArea = new TH2F("hNearAmplitudeVsNearArea","Near amplitude vs Near area", 1024, 0.0, 1024, 1500, 0.0, 15000.0);
   fHistoList->Add(hNearAmplitudeVsNearArea);


   
   // trigger far
   hFarAmplitudeVsPickupPeakPosition = new TH2F("hFarAmplitudeVsPickupPeakPosition","Amplitude Far vs Pickup peak position", 1024, 0.0, 1024,  512, 0.0, 512.0);
   fHistoList->Add(hFarAmplitudeVsPickupPeakPosition);

   hFarTotalAmplitudeVsPickupPeakPosition = new TH2F("hFarTotalAmplitudeVsPickupPeakPosition","(Amplitude+Offset) Far vs Pickup peak position", 1024, 0.0, 1024,  512, 0.0, 512.0);
   fHistoList->Add(hFarTotalAmplitudeVsPickupPeakPosition);

   hFarAreaVsPickupPeakPosition = new TH2F("hFarAreaVsPickupPeakPosition","Area Far vs peak position Pickup", 512, 0.0, 512.0, 1500, 0.0, 15000.0);
   fHistoList->Add(hFarAreaVsPickupPeakPosition);
   
   hFarAreaVsPickupArea = new TH2F("hFarAreaVsPickupArea","Area Far vs area Pickup", 1500, 0.0, 15000.0,  1500, 0.0, 15000.0);
   fHistoList->Add(hFarAreaVsPickupArea);
  
   hFarPositionVsPickupPeakPosition = new TH2F("hFarPositionVsPickupPeakPosition","Far peak position vs Pickup peak position",  512, 0.0, 512.0,  512, 0.0, 512.0);
   fHistoList->Add(hFarPositionVsPickupPeakPosition);
 
   hFarWidthVsPickupPeakPosition = new TH2F("hFarWidthVsPickupPeakPosition","Far peak width vs Pickup peak position", 512, 0.0, 512.0, 100, 0.0, 100.0);
   fHistoList->Add(hFarWidthVsPickupPeakPosition);
   
   hFarAmplitudeVsFarArea = new TH2F("hFarAmplitudeVsFarArea","Far amplitude vs Far area", 1500, 0.0, 15000.0, 1024, 0.0, 1024);
   fHistoList->Add(hFarAmplitudeVsFarArea);



   // trigger correlation
   hNearOffsetVsFarOffset = new TH2F("hNearOffsetVsFarOffset","Near peak offset vs Far peak offest", 200, 0.0, 200.0, 200, 0.0, 200.0);
   fHistoList->Add(hNearOffsetVsFarOffset);


   
   // general
   hPOverTemp1VsEField = new TH2F("hPOverTemp1VsEField", "P/T1 vs EField", 2000, -12000.0, -10000.0, 1000, 2.0, 4.0);
   fHistoList->Add(hPOverTemp1VsEField);

   hPOverTemp2VsEField = new TH2F("hPOverTemp2VsEField", "P/T2 vs EField", 2000, -12000.0, -10000.0, 1000, 2.0, 4.0);
   fHistoList->Add(hPOverTemp2VsEField);

   hTemp1VsEField = new TH2F("hTemp1VsEField", "T1 vs EField", 2000, -12000.0, -10000.0, 100, 250.0, 350.0);
   fHistoList->Add(hTemp1VsEField);

   hTemp2VsEField = new TH2F("hTemp2VsEField", "T2 vs EField", 2000, -12000.0, -10000.0, 100, 250.0, 350.0);
   fHistoList->Add(hTemp2VsEField);

   hTemp1VsTemp2 = new TH2F("hTemp1VsTemp2","T1 vs T2", 100, 250.0, 350.0, 100, 250.0, 350.0);
   fHistoList->Add(hTemp1VsTemp2);
      
   hErrPeakPosition = new TH1F("hErrPeakPosition","Fit error of Pcikup position", 800, -0.5, 5.5);
   fHistoList->Add(hErrPeakPosition);

   hTriggerElecVsTriggerSoft = new TH2F("hTriggerElecVsTriggerSoft","Trigger Bit from Electronic vs Trigger extacted from spectrum", 5, -0.5, 4.5, 5, -0.5, 4.5);
   fHistoList->Add(hTriggerElecVsTriggerSoft);

   hNumberOfEventsPerPack = new TH1F("hNumberOfEventsPerPack","hNumberOfEventsPerPack", 1500, 0.0, 3000.0);
   fHistoList->Add(hNumberOfEventsPerPack);

   hDriftVelocityResolution = new TH1F("hDriftVelocityResolution","# of counts vs drift velocity", 10000, -0.5, 10.5);
   fHistoList->Add(hDriftVelocityResolution);
   
   hNearGainResolution = new TH1F("hNearGainResolution","# of counts vs near gain", 30000, 1000.5, 10000.5);
   fHistoList->Add(hNearGainResolution);
   
   hFarGainResolution = new TH1F("hFarGainResolution","# of counts vs far gain", 30000, 1000.5, 10000.5);
   fHistoList->Add(hFarGainResolution);
}


void CHistoCollection::CreateTimeDependendHistograms(Int_t _NumberOfPacks)
{
   // pack results
   hNearGainArea = new TH1F("hNearGainArea","Gain from Near vs Time", _NumberOfPacks, 0.5, double(_NumberOfPacks)+0.5);
   fHistoList->Add(hNearGainArea);
   
   hFarGainArea  = new TH1F("hFarGainArea","Gain from Far vs Time", _NumberOfPacks, 0.5, double(_NumberOfPacks)+0.5);
   fHistoList->Add(hFarGainArea);
   
   hNearGainAreaCorrected = new TH1F("hNearGainAreaCorrected","Corrected gain from Near vs Time",  _NumberOfPacks, 0.5, double(_NumberOfPacks)+0.5);
   fHistoList->Add(hNearGainAreaCorrected);
   
   hFarGainAreaCorrected  = new TH1F("hFarGainAreaCorrected","Corrected gain from Far vs Time",  _NumberOfPacks, 0.5, double(_NumberOfPacks)+0.5);
   fHistoList->Add(hFarGainAreaCorrected);
   
   hDriftVelocity = new TH1F("hDriftVelocity","Drift velocity vs Time", _NumberOfPacks, 0.5, double(_NumberOfPacks)+0.5);
   fHistoList->Add(hDriftVelocity);

   hDriftVelocityCorrected = new TH1F("hDriftVelocityCorrected","Corrected drift velocity vs Time", _NumberOfPacks, 0.5, double(_NumberOfPacks)+0.5);
   fHistoList->Add(hDriftVelocityCorrected);
   
   hFieldCageHV = new TH1F("hFieldCageHV","Drift voltage vs Time", _NumberOfPacks, 0.5, double(_NumberOfPacks)+0.5);
   fHistoList->Add(hFieldCageHV);

   hPickupHV = new TH1F("hPickupHV","Pickup anode voltage vs Time", _NumberOfPacks, 0.5, double(_NumberOfPacks)+0.5);
   fHistoList->Add(hPickupHV);
   
   hTempSensor1 = new TH1F("hTempSensor1","Temperature from sensor 1 vs Time", _NumberOfPacks, 0.5, double(_NumberOfPacks)+0.5);
   fHistoList->Add(hTempSensor1);

   hTempSensor2 = new TH1F("hTempSensor2","Temperature from sensor 2 vs Time", _NumberOfPacks, 0.5, double(_NumberOfPacks)+0.5);
   fHistoList->Add(hTempSensor2);
   
   hPressure = new TH1F("hPressure","Atmospheric pressure vs Time", _NumberOfPacks, 0.5, double(_NumberOfPacks)+0.5);
   fHistoList->Add(hPressure);
   
   hFractionCO2 = new TH1F("hFractionCO2","CO2 concentraction vs Time", _NumberOfPacks, 0.5, double(_NumberOfPacks)+0.5);
   fHistoList->Add(hFractionCO2);

   hFractionN2 = new TH1F("hFractionN2","N2 concentraction vs Time", _NumberOfPacks, 0.5, double(_NumberOfPacks)+0.5);
   fHistoList->Add(hDriftVelocityCorrected);
   
   hCorrectionFactor_DriftVelocity = new TH1F("hCorrectionFactor_DriftVelocity","Correction factor for drift velocity vs Time", _NumberOfPacks, 0.5, double(_NumberOfPacks)+0.5);
   fHistoList->Add(hCorrectionFactor_DriftVelocity);
      
   hCorrectionFactor_P_TU = new TH1F("hCorrectionFactor_P_TU","Correction factor #frac{P}{T*U} vs Time", _NumberOfPacks, 0.5, double(_NumberOfPacks)+0.5);
   fHistoList->Add(hCorrectionFactor_P_TU);

   hCorrectionFactor_P_T = new TH1F("hCorrectionFactor_P_T","Correction factor #frac{P}{T} vs Time", _NumberOfPacks, 0.5, double(_NumberOfPacks)+0.5);
   fHistoList->Add(hCorrectionFactor_P_T);
}
