//----------------------------------------------------------------------------
//  Header: CHistoCollection.h, v 1.0                         2008/11/19    //
//                                                                          //
//  Goofie ALICE Program Library                                            //
//  Copyright (C) ALICE collaboration, 2008                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#ifndef CHISTOCOLLECTION
#define CHISTOCOLLECTION

class TFile;
class TObject;
class Tlist;
class TH1D;
class TH2D;

class CHistoCollection
{
public:
   CHistoCollection(TFile *_fp);
   ~CHistoCollection();
   
   void     SaveListToFile();
   void     AddObjectToBeSaved(TObject *_save);
   void     CreateClientHistograms();
   void     CreateTimeDependendHistograms(Int_t _NumberOfPacks);

public:
   // list of client histograms
   // pickup
   TH2F    *hAmplitudeVsPeakPos;
   TH2F    *hWidthVsPeakPos;
   TH2F    *hOffsetVsPeakPos;
   TH2F    *hChi2VsPeakPos;
   TH2F    *hAreaVsPeakPos;
   TH2F    *hAmplitudeVsAreaPickup;
   TH2F    *hPOverTemp1VsPickupPeakPosition;
   TH2F    *hPOverTemp2VsPickupPeakPosition;

   
   // trigger near
   TH2F    *hNearAmplitudeVsPickupPeakPosition;
   TH2F    *hNearTotalAmplitudeVsPickupPeakPosition;
   TH2F    *hNearAreaVsPickupPeakPosition;
   TH2F    *hNearAreaVsPickupArea;
   TH2F    *hNearPositionVsPickupPeakPosition;
   TH2F    *hNearWidthVsPickupPeakPosition;
   TH2F    *hNearAmplitudeVsNearArea;

   // trigger far
   TH2F    *hFarAmplitudeVsPickupPeakPosition;
   TH2F    *hFarTotalAmplitudeVsPickupPeakPosition;
   TH2F    *hFarAreaVsPickupPeakPosition;
   TH2F    *hFarAreaVsPickupArea;
   TH2F    *hFarPositionVsPickupPeakPosition;
   TH2F    *hFarWidthVsPickupPeakPosition;
   TH2F    *hFarAmplitudeVsFarArea;

   // trigger correlation
   TH2F    *hNearOffsetVsFarOffset;

   // general
   TH2F    *hPOverTemp1VsEField;
   TH2F    *hPOverTemp2VsEField;
   TH2F    *hTemp1VsEField;
   TH2F    *hTemp2VsEField;
   TH2F    *hTemp1VsTemp2;
   TH1F    *hErrPeakPosition;
   TH2F    *hTriggerElecVsTriggerSoft;
   TH1F    *hNumberOfEventsPerPack;
   TH1F    *hDriftVelocityResolution;
   TH1F    *hFarGainResolution;
   TH1F    *hNearGainResolution;

   
   // pack results
   TH1F    *hNearGainArea;
   TH1F    *hFarGainArea;
   TH1F    *hNearGainAreaCorrected;
   TH1F    *hFarGainAreaCorrected;
   TH1F    *hDriftVelocity;
   TH1F    *hDriftVelocityCorrected;
   TH1F    *hFieldCageHV;
   TH1F    *hPickupHV;
   TH1F    *hTempSensor1;
   TH1F    *hTempSensor2;
   TH1F    *hPressure;
   TH1F    *hFractionCO2;
   TH1F    *hFractionN2;
   TH1F    *hCorrectionFactor_DriftVelocity;
   TH1F    *hCorrectionFactor_P_TU;
   TH1F    *hCorrectionFactor_P_T;
   

private:
   TFile   *fout;
   TList   *fHistoList;   
};
#endif
