#ifndef __CINT__
#include <cstdlib>
#include <iostream>
#include <fstream>
#include <cmath>
#include <deque>
#include <list>
#include <unistd.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <TROOT.h>
#include <TFile.h>
#include <TTree.h>
#include <TStyle.h>
#include <TString.h>
#include <TH1.h>
#include <TF1.h>
#include <TH2.h>
#include <TStopwatch.h>
#include <TTimeStamp.h>

// include header file for goofie root data
#include "AliGoofieRawEvent.h"
#include "AliGoofieGasResult.h"
#include "AliGoofiePack.h"
#include "AliGoofieGasAnalysis.h"

#include "AliGoofieEventInfo.h"
#include "AliGoofiePairAnalysis.h"

#include "CHistoCollection.h"


using namespace std;
#endif

int loop(TString input_ff, TString output_ff, TString _configFile);
list<TString> *TakeListOfFiles(TString  *fpathName);
TH1D *MakeUniqueHistogramName(TH1D &_histo, TString _FileName);
unsigned long GetTotalNumberOfPackForAnalysis(list<TString> *_FileList, AliGoofiePack *_fpack, const int &_PacksPerEvents);

#ifndef __CINT__
int main(int argc, char *argv[])
{
   TString FileInput;
   TString FileOutput;
   TString ConfigFile("");
   Int_t   ok=-1;
   
   if(argv[1] != NULL) {
      FileInput = argv[1];
      if(argv[2] != NULL) {
	 FileOutput = argv[2];
	 if(argv[3] != NULL) {
	   ConfigFile = argv[3];
	 }
	 ok = loop(FileInput, FileOutput, ConfigFile);
      } 
   }
   if(ok != 0) {
     cout << "Usage: " << argv[0] << " [inputFiles] [outputFile] [configFile]" << endl;
      cout << "    where  inputFiles is a path where Goofie's root files are located \n"
	   << "    where  outputFile is the path and the name of file where the result\n" 
	   << "                      of the application should be save\n"
	   << "    where  configFile is a location to the configuration file\n" <<endl; 
   }
   return 0;
}
#endif
 

int loop(TString input_ff, TString output_ff, TString _configFile)
{
   // flags
   //
   Bool_t fSingleSpectraSaved = false;   // single event distributions -- only for tests, take into account huge output file
   Bool_t fPackSpectraSaved   = true;    // save in output file pack distributions -- gain, extracted time, fit results
   Int_t  printoutFrequency   = 1;  
   unsigned long CurrentBin   = 0;
   unsigned int fEventUsedForAnalysis = 2500;

   TString fLocationOfConfigurationFile = "/home/goofie/code/offLine/config/softwareTPCsetup.conf";
   if(_configFile.Length()>0) {
     fLocationOfConfigurationFile = _configFile;
   }

    cout << "config = " << fLocationOfConfigurationFile << endl;

   // Create Gas analysis objects
   AliGoofieGasResult   *fGasResults = new AliGoofieGasResult();
   AliGoofieGasAnalysis *fGasAna     = new AliGoofieGasAnalysis(fGasResults);
   // set configuration file name and load it
   fGasAna->SetConfiguration(fLocationOfConfigurationFile.Data());
   // start directory
   TDirectory *fStartDirectory = gDirectory;

   // time counter
   TStopwatch timer;                   // Timer counter
   timer.Start();
   // private option - be carefully why modifying them
   Int_t  entryOk          = 0;
   
   // Create Output File with histograms
   TFile *fout = new TFile(output_ff.Data(), "RECREATE");
   if(fout->IsZombie() || !fout->IsOpen()) {
      return -100;
   }
   // Create Histograms
   CHistoCollection *fHisto = new CHistoCollection(fout);
  
   // ROOT File objects handlers
   TFile *hfile = NULL;
   TTree *tree  = NULL;

   // Get List of root files in directory
   list<TString> *listOfFiles = TakeListOfFiles(&input_ff);
   if(listOfFiles == NULL) {
    return -102;
   }
   unsigned int sizeList = listOfFiles->size();
   cout << "Total number of file to be analyzed = " << sizeList << endl;

   //Raw Data Array
   std::deque<AliGoofieRawEvent> fRAWDataInput;
   //Set address of CPack
   AliGoofiePack *fpack = new AliGoofiePack();   //we create the event object once 
 
   // Get Number of pack that will be analyzed from the data set
   unsigned long numberOfPackToAnalyze = GetTotalNumberOfPackForAnalysis(listOfFiles, fpack, fEventUsedForAnalysis);
  
   // create histograms for gas analysis results
   cout << "Create Histograms vs Time for " << numberOfPackToAnalyze << " packs" << endl;
   fHisto->CreateTimeDependendHistograms(numberOfPackToAnalyze);
   
   //loop over input files
   for (unsigned int fNr = 0; fNr < sizeList; ++fNr) {
      // get input file
      TString inputFile = listOfFiles->front();
      cout <<  "no. = "<< fNr << ", Input file = " << inputFile << endl;
      listOfFiles->pop_front();

      //Open File  
      hfile = new TFile(inputFile.Data(), "READ");
      if(hfile->IsZombie()) {
	 cerr << "Error in <loop()>: Fill is not readable, skipping file" << flush << endl;
	 if(hfile) {
	    delete hfile;
	    hfile = NULL;
	 }
	 continue; 
      }
      tree = (TTree*)hfile->Get("T");
      tree->SetBranchAddress("Packs", &fpack);
      tree->SetBranchStatus("*", 1);
      
      Int_t npack = (Int_t) tree->GetEntries();
     
      cout << "Number of packs pre file = " << npack << endl;
  
      for (Int_t ev=0; ev<npack; ++ev) {
	 // *************** main loop ***************
	 // for every pack one need to clean memory first -- ROOT way of processing events
	 fpack->Clear();
	 
	 entryOk = tree->GetEvent(ev);
	 if(entryOk < 0) {
	    cout << "Error event number = " <<  ev << endl; 
	    continue;  
	 }
	 if(ev%printoutFrequency == 0) {
	    cout<<">>>>==============-======== START analyzing pack no. :: " << ev <<endl;
	    cout << "number of events per pack : " << fpack->GetNumberOfEvents() << endl;
	 } 
	 
	 // loop over events and fill input array - to interface onLine analysis code
	 for(int eventID=0; eventID<fpack->GetNumberOfEvents(); ++eventID) {
            AliGoofieRawEvent *event = fpack->GetCRawEvent(eventID);
	    // fill list
	    fRAWDataInput.push_back(*event);
	 }
	 
	 // add required number of events
	 if(fRAWDataInput.size() < fEventUsedForAnalysis) continue;
	  
	 //---------------------------- Start Gas Analysis -------------------------------//  
	 TDirectory *lCurrentDirecotry = gDirectory;
	 gDirectory = fStartDirectory;
     	 fGasAna->AnalyzePack(fRAWDataInput);
	 // print results
	 fGasAna->PrintResults(true);
	 
	 //-------------------- fill pack histograms -------------------------------------//
	 cout << "current pack index = " << ++CurrentBin << endl;  
	 
     	 fHisto->hNearGainArea->SetBinContent(CurrentBin, fGasAna->GetGasAnaResults()->fGainNear);
	 fHisto->hFarGainArea->SetBinContent(CurrentBin, fGasAna->GetGasAnaResults()->fGainFar);
	 fHisto->hNearGainAreaCorrected->SetBinContent(CurrentBin, fGasAna->GetGasAnaResults()->fGainNearCorrected);
	 fHisto->hFarGainAreaCorrected->SetBinContent(CurrentBin, fGasAna->GetGasAnaResults()->fGainFarCorrected);
	 fHisto->hDriftVelocity->SetBinContent(CurrentBin, fGasAna->GetGasAnaResults()->fDriftVelocity);
	 fHisto->hDriftVelocityCorrected->SetBinContent(CurrentBin, fGasAna->GetGasAnaResults()->fDriftVelocityCorrected);
	 fHisto->hFieldCageHV->SetBinContent(CurrentBin, fGasAna->GetGasAnaResults()->fFieldCageHV);
	 fHisto->hPickupHV->SetBinContent(CurrentBin, fGasAna->GetGasAnaResults()->fPickupHV);
	 fHisto->hTempSensor1->SetBinContent(CurrentBin, fGasAna->GetGasAnaResults()->fTemp1);
	 fHisto->hTempSensor2->SetBinContent(CurrentBin, fGasAna->GetGasAnaResults()->fTemp2);
	 fHisto->hPressure->SetBinContent(CurrentBin, fGasAna->GetGasAnaResults()->fPressure);
	 fHisto->hFractionCO2->SetBinContent(CurrentBin, fGasAna->GetGasAnaResults()->fFractionCO2);
	 fHisto->hFractionN2->SetBinContent(CurrentBin, fGasAna->GetGasAnaResults()->fFractionN2);
         fHisto->hCorrectionFactor_DriftVelocity->SetBinContent(CurrentBin, fGasAna->GetGasAnaResults()->fCorrectionFactor);
         if(fGasAna->GetGasAnaResults()->fTemp2) {
	    fHisto->hCorrectionFactor_P_TU->SetBinContent(CurrentBin, (fGasAna->GetGasAnaResults()->fPressure/
	                                                               fabs(fGasAna->GetGasAnaResults()->fTemp2*fGasAna->GetGasAnaResults()->fFieldCageHV)));
    	    fHisto->hCorrectionFactor_P_T->SetBinContent(CurrentBin, fGasAna->GetGasAnaResults()->fPressure/fGasAna->GetGasAnaResults()->fTemp2);
	 }
	 // info
	 fHisto->hNumberOfEventsPerPack->Fill(fGasAna->GetPairInfoPtr()->GetCurrentNumberOfUsedEvents());
	 // resolution
	 fHisto->hDriftVelocityResolution->Fill(fGasAna->GetGasAnaResults()->fDriftVelocityCorrected);
	 fHisto->hFarGainResolution->Fill(fGasAna->GetGasAnaResults()->fGainFarCorrected);
	 fHisto->hNearGainResolution->Fill(fGasAna->GetGasAnaResults()->fGainNearCorrected);
	 
	 if(fPackSpectraSaved) {
	    if(fGasAna->GetPairInfoPtr()->fTimePickupSpectrum) {
	       fHisto->AddObjectToBeSaved(MakeUniqueHistogramName(*fGasAna->GetPairInfoPtr()->fTimePickupSpectrum, inputFile));
	    }
	    if(fGasAna->GetPairInfoPtr()->fTimePickupSpectrumAll) {
	       fHisto->AddObjectToBeSaved(MakeUniqueHistogramName(*fGasAna->GetPairInfoPtr()->fTimePickupSpectrumAll, inputFile));
	    }
	    // fit results
	    TF1 *tmp = fGasAna->GetPairInfoPtr()->GetPrtToDriftVelocityAnalysis()->MakeNearFitFunction(fGasAna->GetFitFunctionPrt());
	    TString name = tmp->GetName();
	    name += inputFile;
	    tmp->SetName(name.Data());
	    fHisto->AddObjectToBeSaved(tmp);

	    cout << "TF1 near name = " << tmp->GetName() << endl;
	        
	    tmp = fGasAna->GetPairInfoPtr()->GetPrtToDriftVelocityAnalysis()->MakeFarFitFunction(fGasAna->GetFitFunctionPrt());
	    name = tmp->GetName();
	    name += inputFile;
	    tmp->SetName(name.Data());
	    fHisto->AddObjectToBeSaved(tmp);

	    cout << "TF1 far name = " << tmp->GetName() << endl;
	    
	    if(fGasAna->GetPairInfoPtr()->fAreaSpectrumTByNear) {
	       fHisto->AddObjectToBeSaved(MakeUniqueHistogramName(*fGasAna->GetPairInfoPtr()->fAreaSpectrumTByNear, inputFile));
	    }
	    if(fGasAna->GetPairInfoPtr()->fAreaSpectrumTByFar) {
	       fHisto->AddObjectToBeSaved(MakeUniqueHistogramName(*fGasAna->GetPairInfoPtr()->fAreaSpectrumTByFar, inputFile));
	    }
	    if(fGasAna->GetPairInfoPtr()->fAmplSpectrumTByNear) {
	       fHisto->AddObjectToBeSaved(MakeUniqueHistogramName(*fGasAna->GetPairInfoPtr()->fAmplSpectrumTByNear, inputFile));
	    }
	    if(fGasAna->GetPairInfoPtr()->fAmplSpectrumTByFar) {
	       fHisto->AddObjectToBeSaved(MakeUniqueHistogramName(*fGasAna->GetPairInfoPtr()->fAmplSpectrumTByFar, inputFile));
	    }
	 }
	 //----------------- Add some histograms to file from current pack --------------//
	 for(int eventID=0; eventID<fGasAna->GetEventListSize(); ++eventID) {
	 
	    // single event distribution -- only for tests
	    if(fSingleSpectraSaved) {
	       fHisto->AddObjectToBeSaved(fGasAna->GetEventFromPack(eventID)->MakePickupSpectrum());
	       fHisto->AddObjectToBeSaved(fGasAna->GetEventFromPack(eventID)->MakePickupFitFunction(fGasAna->GetFitFunctionPrt()));
	       
	       fHisto->AddObjectToBeSaved(fGasAna->GetEventFromPack(eventID)->MakeNearSpectrum());
	       fHisto->AddObjectToBeSaved(fGasAna->GetEventFromPack(eventID)->MakeNearFitFunction(fGasAna->GetFitFunctionPrt()));
	       
	       fHisto->AddObjectToBeSaved(fGasAna->GetEventFromPack(eventID)->MakeFarSpectrum());
	       fHisto->AddObjectToBeSaved(fGasAna->GetEventFromPack(eventID)->MakeFarFitFunction(fGasAna->GetFitFunctionPrt()));
	    }
	    // data acumulated over many events
	    // fill some histogram created in CHistoCollection
	    fHisto->hTriggerElecVsTriggerSoft->Fill(fGasAna->GetEventFromPack(eventID)->fPeakTriggeredBy,
						    fGasAna->GetEventFromPack(eventID)->GetTriggerChannel());
	    
	    // if good trigger
	    if(fGasAna->GetEventFromPack(eventID)->fPeakTriggeredBy > TG_NOISE) { 
	       fHisto->hAmplitudeVsPeakPos->Fill(fGasAna->GetEventFromPack(eventID)->fPickupPeakInfo.fPosition,
						 fGasAna->GetEventFromPack(eventID)->fPickupPeakInfo.fAmplitude);
	       fHisto->hWidthVsPeakPos->Fill(fGasAna->GetEventFromPack(eventID)->fPickupPeakInfo.fPosition,
					     fGasAna->GetEventFromPack(eventID)->fPickupPeakInfo.fWidth);
	       fHisto->hOffsetVsPeakPos->Fill(fGasAna->GetEventFromPack(eventID)->fPickupPeakInfo.fPosition,
					      fGasAna->GetEventFromPack(eventID)->fPickupPeakInfo.fOffset);
	       fHisto->hChi2VsPeakPos->Fill(fGasAna->GetEventFromPack(eventID)->fPickupPeakInfo.fPosition,
					    fGasAna->GetEventFromPack(eventID)->fPickupPeakInfo.fChi2);
	       fHisto->hAreaVsPeakPos->Fill(fGasAna->GetEventFromPack(eventID)->fPickupPeakInfo.fPosition,
					    fGasAna->GetEventFromPack(eventID)->fPickupPeakInfo.fArea);
	       fHisto->hAmplitudeVsAreaPickup->Fill(fGasAna->GetEventFromPack(eventID)->fPickupPeakInfo.fArea,
						    fGasAna->GetEventFromPack(eventID)->fPickupPeakInfo.fAmplitude);
	       
	       if(fGasAna->GetEventFromPack(eventID)->GetTemp1() != 0.0) {
		  
		  fHisto->hPOverTemp1VsPickupPeakPosition->Fill(fGasAna->GetEventFromPack(eventID)->fPickupPeakInfo.fPosition,
								fGasAna->GetEventFromPack(eventID)->fSlowControl.sPress/fGasAna->GetEventFromPack(eventID)->GetTemp1());
		  
		  fHisto->hPOverTemp1VsEField->Fill(fGasAna->GetEventFromPack(eventID)->fSlowControl.sEfVoltage,
						    fGasAna->GetEventFromPack(eventID)->fSlowControl.sPress/fGasAna->GetEventFromPack(eventID)->GetTemp1());
		 
	       }
	       	
	       if(fGasAna->GetEventFromPack(eventID)->GetTemp2() != 0.0) {
		  fHisto->hPOverTemp2VsPickupPeakPosition->Fill(fGasAna->GetEventFromPack(eventID)->fPickupPeakInfo.fPosition,
								fGasAna->GetEventFromPack(eventID)->fSlowControl.sPress/fGasAna->GetEventFromPack(eventID)->GetTemp2());
		  		    
		  fHisto->hPOverTemp2VsEField->Fill(fGasAna->GetEventFromPack(eventID)->fSlowControl.sEfVoltage,
						    fGasAna->GetEventFromPack(eventID)->fSlowControl.sPress/fGasAna->GetEventFromPack(eventID)->GetTemp2());
	       }
	       
	       fHisto->hTemp1VsEField->Fill(fGasAna->GetEventFromPack(eventID)->fSlowControl.sEfVoltage,
					    fGasAna->GetEventFromPack(eventID)->GetTemp1());
	       
	       fHisto->hTemp2VsEField->Fill(fGasAna->GetEventFromPack(eventID)->fSlowControl.sEfVoltage,
					    fGasAna->GetEventFromPack(eventID)->GetTemp2());
	       
	       fHisto->hTemp1VsTemp2->Fill(fGasAna->GetEventFromPack(eventID)->GetTemp2(), fGasAna->GetEventFromPack(eventID)->GetTemp1());
	       
	       fHisto->hErrPeakPosition->Fill(fGasAna->GetEventFromPack(eventID)->fPickupPeakInfo.fErrPosition);
	       
	       fHisto->hNearOffsetVsFarOffset->Fill(fGasAna->GetEventFromPack(eventID)->fTriggerFarPeakInfo.fOffset,
						    fGasAna->GetEventFromPack(eventID)->fTriggerNearPeakInfo.fOffset);
	       
	       
	       if(fGasAna->GetEventFromPack(eventID)->fPeakTriggeredBy == TG_FAR) {     // FAR TRIGGER
		  fHisto->hFarAmplitudeVsPickupPeakPosition->Fill(fGasAna->GetEventFromPack(eventID)->fPickupPeakInfo.fPosition,
								  fGasAna->GetEventFromPack(eventID)->fTriggerFarPeakInfo.fAmplitude);
		 
		 fHisto->hFarTotalAmplitudeVsPickupPeakPosition->Fill(fGasAna->GetEventFromPack(eventID)->fPickupPeakInfo.fPosition,
								     (fGasAna->GetEventFromPack(eventID)->fTriggerFarPeakInfo.fOffset +
								      fGasAna->GetEventFromPack(eventID)->fTriggerFarPeakInfo.fAmplitude));
		  
		  
		  fHisto->hFarAreaVsPickupPeakPosition->Fill(fGasAna->GetEventFromPack(eventID)->fPickupPeakInfo.fPosition,
							     fGasAna->GetEventFromPack(eventID)->fTriggerFarPeakInfo.fArea);
		  
		  fHisto->hFarAreaVsPickupArea->Fill(fGasAna->GetEventFromPack(eventID)->fPickupPeakInfo.fArea,
						     fGasAna->GetEventFromPack(eventID)->fTriggerFarPeakInfo.fArea);
		  
		  
		  fHisto->hFarPositionVsPickupPeakPosition->Fill(fGasAna->GetEventFromPack(eventID)->fPickupPeakInfo.fPosition,
								 fGasAna->GetEventFromPack(eventID)->fTriggerFarPeakInfo.fPosition);
		  
		  fHisto->hFarWidthVsPickupPeakPosition->Fill(fGasAna->GetEventFromPack(eventID)->fPickupPeakInfo.fPosition,
							      fGasAna->GetEventFromPack(eventID)->fTriggerFarPeakInfo.fWidth);
		  
		  fHisto->hFarAmplitudeVsFarArea->Fill(fGasAna->GetEventFromPack(eventID)->fTriggerFarPeakInfo.fArea,
						       fGasAna->GetEventFromPack(eventID)->fTriggerFarPeakInfo.fAmplitude);
		  
	       }
	       
	       if(fGasAna->GetEventFromPack(eventID)->fPeakTriggeredBy == TG_NEAR) {     // NEAR TRIGGER
		  fHisto->hNearAmplitudeVsPickupPeakPosition->Fill(fGasAna->GetEventFromPack(eventID)->fPickupPeakInfo.fPosition,
								   fGasAna->GetEventFromPack(eventID)->fTriggerNearPeakInfo.fAmplitude);
		  
		   fHisto->hNearTotalAmplitudeVsPickupPeakPosition->Fill(fGasAna->GetEventFromPack(eventID)->fPickupPeakInfo.fPosition,
							                 (fGasAna->GetEventFromPack(eventID)->fTriggerNearPeakInfo.fOffset +
								          fGasAna->GetEventFromPack(eventID)->fTriggerNearPeakInfo.fAmplitude));
		  
		  fHisto->hNearAreaVsPickupPeakPosition->Fill(fGasAna->GetEventFromPack(eventID)->fPickupPeakInfo.fPosition,
							      fGasAna->GetEventFromPack(eventID)->fTriggerNearPeakInfo.fArea);
		  
		  fHisto->hNearAreaVsPickupArea->Fill(fGasAna->GetEventFromPack(eventID)->fPickupPeakInfo.fArea,
						      fGasAna->GetEventFromPack(eventID)->fTriggerNearPeakInfo.fArea);
		  
		  fHisto->hNearPositionVsPickupPeakPosition->Fill(fGasAna->GetEventFromPack(eventID)->fPickupPeakInfo.fPosition,
								  fGasAna->GetEventFromPack(eventID)->fTriggerNearPeakInfo.fPosition);
		  
		  fHisto->hNearWidthVsPickupPeakPosition->Fill(fGasAna->GetEventFromPack(eventID)->fPickupPeakInfo.fPosition,
							       fGasAna->GetEventFromPack(eventID)->fTriggerNearPeakInfo.fWidth);
		  
		  fHisto->hNearAmplitudeVsNearArea->Fill(fGasAna->GetEventFromPack(eventID)->fTriggerNearPeakInfo.fAmplitude,
							 fGasAna->GetEventFromPack(eventID)->fTriggerNearPeakInfo.fArea);
		  
	       } 
	    }
	    gDirectory = lCurrentDirecotry;
	 }
         // clear output event list
	 fGasAna->ClearEventList();
	 // clear raw event list
	 fRAWDataInput.clear();
	 
	 cout << "--------------- after loop ------------------- " << endl;
      }
      // close input file
      if(hfile) {
	 hfile->Close();
	 delete hfile;
	 hfile = NULL;
      }
   }
   // set main save directory
   fout->cd();
     
   // Add global histograms to be saved
   fHisto->AddObjectToBeSaved(fGasAna->GetPairInfoPtr()->GetPrtToDriftVelocityAnalysis()->GetPrtToDriftVelocityVsGasDensity());
   fHisto->AddObjectToBeSaved(fGasAna->GetPairInfoPtr()->GetPrtToGasCompositionAnalysis()->GetPrtToGainNearVsGasDensity());
   fHisto->AddObjectToBeSaved(fGasAna->GetPairInfoPtr()->GetPrtToGasCompositionAnalysis()->GetPrtToGainFarVsGasDensity());
   
   cout << "Mean P/TU   value = " << fGasAna->GetPairInfoPtr()->GetPrtToDriftVelocityAnalysis()->GetMeanP_TU() << endl;
   cout << "Mean P/T    value = " << fGasAna->GetPairInfoPtr()->GetPrtToGasCompositionAnalysis()->GetMeanP_T() << endl;
   cout << "Mean DriftV value = " << fGasAna->GetPairInfoPtr()->GetPrtToDriftVelocityAnalysis()->GetMeanDriftVelocity() << endl;
   cout << "Mean Gain N value = " << fGasAna->GetPairInfoPtr()->GetPrtToGasCompositionAnalysis()->GetMeanGasGainNear() << endl;
   cout << "Mean Gain F value = " << fGasAna->GetPairInfoPtr()->GetPrtToGasCompositionAnalysis()->GetMeanGasGainFar() << endl;
  
   timer.Stop();
   Double_t rtime = timer.RealTime();
   Double_t ctime = timer.CpuTime();
   cout << "RealTime="<< rtime << " seconds, CpuTime=" <<ctime<< " seconds" << endl;
   cout<<"done with the loop"<<endl;    

   // Write histograms to file
   fHisto->SaveListToFile();
   // Close File
   cout<<"after 1st histo"<<endl;
   cout<<"Going to close root HISTO file"<<endl;
   fout->Close();

   // deleteing global objects

   cout << "Going to delete DeleteTSpectrumObj " << flush << endl;
   
   AliGoofiePeakProperties::DeleteTSpectrumObj();

   cout << "delete rest " << flush << endl;
   
   // deleting local objects
   if(fout) {
      delete fout;
      fout = NULL;
   }

   cout << "after TFile delete" << flush << endl;
  
   if(fGasResults) {
      delete fGasResults;
      fGasResults = NULL;
   }

   cout << "after AliGoofieGasResult delete" << flush << endl;
  

   if(fGasAna) {
      //delete fGasAna;
      //fGasAna = NULL;
   }

   cout << "after AliGoofieGasAnalysis delete" << flush << endl;
 
   if(fpack) {
      delete fpack;
      fpack = NULL;
   }

   cout << "after AliGoofiePack delete" << flush << endl;
 
   if(fHisto) {
      delete fHisto;
      fHisto = NULL;
   }
   return 0;
}


TH1D *MakeUniqueHistogramName(TH1D &_histo, TString _FileName)
{
   TH1D *temp = new TH1D(_histo);
   TString name = temp->GetName();
   name += "_"; name += _FileName;
   temp->SetName(name.Data());
   return temp;
}

list<TString> *TakeListOfFiles(TString  *fpathName)
{
   DIR            *dir          = NULL;
   struct  dirent *dir_cont     = NULL;
   struct  stat    st;
   list<TString>  *flistOfFiles = new list<TString>;

   TString fname     = "/";
   // check for '/' in the fpathName
   if(fname.CompareTo(fpathName->operator[](fpathName->Length()-1))!=0) {
      fpathName->Append("/");
   }
   dir = opendir(fpathName->Data());
   if(dir) {
      while((dir_cont=readdir(dir))!=NULL) {
         fname = *fpathName + dir_cont->d_name;
         lstat(fname.Data(), &st);
         if(S_ISREG(st.st_mode) || S_ISLNK(st.st_mode)) {
	    // check if the name contain .root extension
	    if(fname.Index(".root")>=0) {
	       flistOfFiles->push_back(fname);
	    }
         }
      }
      closedir(dir);
      //set list in increasing order
      flistOfFiles->sort();
   } else {
      cerr << "Error in <TakeListOfFiles>: list of file is missing " << flush << endl;
      return NULL;
   }
   return flistOfFiles;
}


unsigned long GetTotalNumberOfPackForAnalysis(list<TString> *_FileList, AliGoofiePack *_fpack, const int &_PacksPerEvents) 
{
  // save current directory
  TDirectory    *fSavedHisto           = gDirectory;
  unsigned long  numberOfEventsPerPack = 0; 
  
  //loop over input files
  for (list<TString>::iterator it = _FileList->begin(); it != _FileList->end(); ++it) {
    // get input file
    TString inputFile = *it;
    cout << "Input file = " << inputFile << endl; 
    //Open File  
    TFile *lfile = new TFile(inputFile.Data(), "READ");
    if(lfile->IsZombie()) {
      cerr << "Error in <loop()>: Fill is not readable, skipping file" << flush << endl;
      if(lfile) {
	delete lfile;
	lfile = NULL;
      }
      continue; 
    }
    TTree *ltree = (TTree*)lfile->Get("T");
    ltree->SetBranchAddress("Packs", &_fpack);
    ltree->SetBranchStatus("*", 1);

    for (Int_t ev=0; ev<ltree->GetEntries(); ++ev) {
      // *************** event loop ***************
      // for every pack one need to clean memory first -- ROOT way of processing events
      _fpack->Clear();
      
      if(ltree->GetEvent(ev) < 0) {
	cerr << "Error event number = " <<  ev << endl; 
	continue;  
      }
      // fill list 
      numberOfEventsPerPack += _fpack->GetNumberOfEvents();
    }   
    // close and delete file
    if(lfile) {
      lfile->Close();
      delete lfile;
      lfile = NULL;
    }      
  }
  // set main directory
  gDirectory = fSavedHisto;

  // return value
  if(_PacksPerEvents<=0) 
    return (numberOfEventsPerPack+1);
  else
    return (unsigned long)(1+((float)numberOfEventsPerPack/(float)_PacksPerEvents));
}
