TString GetNumber(double _number, int decimal);

void disp(TString fname)
{
  gStyle->SetPalette(1);
  //gStyle->SetOptStat(0);
  //gStyle->SetOptTitle(0);
  
  TFile *f = new TFile(fname.Data(), "READ");
  f->ls();
  
  TH2D *DriftVelocityVsGasDensity = (TH2D*)f->Get("DriftVelocityVsGasDensity");
  DriftVelocityVsGasDensity->SetFillColor(0);

  TF1 *line = new TF1("line", "[0]+[1]*x",0.9, 1.1);
  line->SetLineColor(2);
  line->SetLineWidth(2);
  

  TCanvas *c = new TCanvas("c", "correlation", 1300, 500);
  c->SetFillColor(0);
  c->Divide(2,1);

  c->cd(1);
  DriftVelocityVsGasDensity->GetXaxis()->SetRangeUser(0.985, 1.03);
  DriftVelocityVsGasDensity->GetXaxis()->SetNdivisions(505);

  DriftVelocityVsGasDensity->GetYaxis()->SetRangeUser(1.7, 2.6);
  
  // X
  DriftVelocityVsGasDensity->SetXTitle("#bar{#rho}=#LTU#upointT/P#GT#upointP/U#upointT");
  // Y
  DriftVelocityVsGasDensity->SetYTitle("v_{drift} (cm/#mus)");
  DriftVelocityVsGasDensity->SetTitleOffset(1.2, "Y");
  // get profile
  DriftVelocityVsGasDensity->ProfileX("DriftVelocityVsGasDensity_profX");
  // fit
  DriftVelocityVsGasDensity_profX->Sumw2();
  DriftVelocityVsGasDensity_profX->Fit("line"); 
  // draw
  DriftVelocityVsGasDensity->Draw("col1z");
  DriftVelocityVsGasDensity_profX->Draw("same");
  
  double    P0   = line->GetParameter(0);
  TString  sP0   = GetNumber(P0, 4);
  double errP0   = line->GetParError(0);
  TString serrP0 = GetNumber(errP0, 1);
  double    P1   = line->GetParameter(1);
  TString  sP1   = GetNumber(P1, 4);
  double errP1   = line->GetParError(1);
  TString serrP1 = GetNumber(errP1, 1);
 
  TString stotP0 = "P_{0} = " + sP0 + " #pm " + serrP0;
  TString stotP1 = "P_{1} = " + sP1 + " #pm " + serrP1;

  TLegend *leg = new TLegend(0.51, 0.79, 0.897, 0.898);
  leg->SetFillColor(0);
  leg->SetBorderSize(0);
  leg->SetTextSize(0.04);
  leg->AddEntry(line, "v_{drift} = P_{0} + P_{1}#upoint#bar{#rho}","lp");
  leg->Draw();
  
  TLatex *info = new TLatex();
  info->SetTextSize(0.04);
  //info->DrawLatex(1.014, 1.98, "v_{drift} = P1 + P2#upoint#bar{#rho}");
  info->DrawLatex(1.014, 2.45,stotP0.Data());
  info->DrawLatex(1.014, 2.4, stotP1.Data());


  c->cd(2);
  
  TH2D *fhGainNearVsGasDensity = (TH2D*)f->Get("fhGainNearVsGasDensity");
  fhGainNearVsGasDensity->SetFillColor(0);
  fhGainNearVsGasDensity->GetXaxis()->SetRangeUser(0.985, 1.03);
  fhGainNearVsGasDensity->GetXaxis()->SetNdivisions(505);
  fhGainNearVsGasDensity->GetYaxis()->SetRangeUser(2000.0, 4000.0);
  fhGainNearVsGasDensity->GetYaxis()->SetNdivisions(505);
  // X
  fhGainNearVsGasDensity->SetXTitle("#bar{#rho}=#LTT/P#GT#upointP/T");
  // Y
  fhGainNearVsGasDensity->SetYTitle("gas gain (a.u.)");
  fhGainNearVsGasDensity->SetTitleOffset(1.3, "Y");
  
  fhGainNearVsGasDensity->Draw("col1z");
     
  TH2D *fhGainFarVsGasDensity = (TH2D*)f->Get("fhGainFarVsGasDensity");
  fhGainFarVsGasDensity->SetFillColor(0);
  fhGainFarVsGasDensity->GetXaxis()->SetRangeUser(0.985, 1.03);
  fhGainFarVsGasDensity->GetXaxis()->SetNdivisions(505);
  fhGainFarVsGasDensity->GetYaxis()->SetRangeUser(2000.0, 4000.0);
  fhGainFarVsGasDensity->Draw("same");
}

TString GetNumber(double _number, int decimal) 
{
  TString s; s += _number;
  int _exp = s.Index("e");
  int _dot = s.Index(".");
  if(_exp>0) {
    TString sExp = s;
    sExp.Remove(0,_exp);
  }
  s.Remove(_dot+decimal+1);
  if(_exp>0) {
    s.Append(sExp);
  }
  return s;
}
