TString GetNumber(double _number, int decimal);

void disp(TString fname)
{
  gStyle->SetPalette(1);
  gStyle->SetOptStat(0);
  gStyle->SetOptTitle(0);

  TFile *f = new TFile(fname.Data(), "READ");

  TH2D *DriftVelocityVsGasDensity = (TH2D*)f->Get("DriftVelocityVsGasDensity");
  DriftVelocityVsGasDensity->SetFillColor(0);

  TF1 *line = new TF1("line", "[0]+[1]*x", 0.98, 1.05);
  line->SetLineColor(2);
  line->SetLineWidth(2);
  line->SetParameters(5.0, -2.0);

  TF1 *lineGainNear = new TF1("lineGainNear", "[0]+[1]*x",0.98, 1.05);
  lineGainNear->SetLineColor(2);
  lineGainNear->SetLineWidth(2);

  TF1 *lineGainFar = new TF1("lineGainFar", "[0]+[1]*x",0.98, 1.05);
  lineGainFar->SetLineColor(4);
  lineGainFar->SetLineWidth(2);


  TCanvas *c = new TCanvas("c", "correlation", 1300, 500);
  c->SetFillColor(0);
  c->Divide(2,1);

  c->cd(1);
  DriftVelocityVsGasDensity->GetXaxis()->SetRangeUser(0.985, 1.03);
  DriftVelocityVsGasDensity->GetXaxis()->SetNdivisions(505);

  DriftVelocityVsGasDensity->GetYaxis()->SetRangeUser(1.5, 4.5);

  // X
  DriftVelocityVsGasDensity->SetXTitle("#bar{#rho}=#LTU#upointT/P#GT#upointP/U#upointT");
  // Y
  DriftVelocityVsGasDensity->SetYTitle("v_{drift} (cm/#mus)");
  DriftVelocityVsGasDensity->SetTitleOffset(1.2, "Y");
  // get profile
  DriftVelocityVsGasDensity->ProfileX("DriftVelocityVsGasDensity_profX");
  // fit
  DriftVelocityVsGasDensity_profX->Sumw2();
  DriftVelocityVsGasDensity_profX->Fit("line","RM");
  // draw
  DriftVelocityVsGasDensity->Draw("col1z");
  DriftVelocityVsGasDensity_profX->Draw("same");

  double    P0   = line->GetParameter(0);
  TString  sP0   = GetNumber(P0, 4);
  double errP0   = line->GetParError(0);
  TString serrP0 = GetNumber(errP0, 1);
  double    P1   = line->GetParameter(1);
  TString  sP1   = GetNumber(P1, 4);
  double errP1   = line->GetParError(1);
  TString serrP1 = GetNumber(errP1, 1);

  TString stotP0 = "P_{0} = " + sP0 + " #pm " + serrP0;
  TString stotP1 = "P_{1} = " + sP1 + " #pm " + serrP1;

  TLegend *leg = new TLegend(0.51, 0.65, 0.897, 0.898);
  leg->SetFillColor(0);
  leg->SetBorderSize(0);
  leg->SetTextSize(0.04);
  leg->AddEntry(line, "v_{drift} = P_{0} + P_{1}#upoint#bar{#rho}","lp");
  leg->AddEntry(NULL, stotP0.Data(), "");
  leg->AddEntry(NULL, stotP1.Data(), "");
  leg->Draw();

  //TLatex *info = new TLatex();
  //info->SetTextSize(0.04);
  //info->DrawLatex(1.014, 1.98, "v_{drift} = P1 + P2#upoint#bar{#rho}");
  //info->DrawLatex(1.014, 2.45,stotP0.Data());
  //info->DrawLatex(1.014, 2.4, stotP1.Data());


  c->cd(2);

  TH2D *fhGainNearVsGasDensity = (TH2D*)f->Get("fhGainNearVsGasDensity");
  fhGainNearVsGasDensity->SetFillColor(0);
  fhGainNearVsGasDensity->GetXaxis()->SetRangeUser(0.985, 1.03);
  fhGainNearVsGasDensity->GetXaxis()->SetNdivisions(505);
  fhGainNearVsGasDensity->GetYaxis()->SetRangeUser(2000.0, 6000.0);
  fhGainNearVsGasDensity->GetYaxis()->SetNdivisions(505);
  // X
  fhGainNearVsGasDensity->SetXTitle("#bar{#rho}=#LTT/P#GT#upointP/T");
  // Y
  fhGainNearVsGasDensity->SetYTitle("gas gain (a.u.)");
  fhGainNearVsGasDensity->SetTitleOffset(1.3, "Y");
  // get profile
  fhGainNearVsGasDensity->ProfileX("fhGainNearVsGasDensity_profX");
  // fit
  cout << "---------------> Gain Near <------------------------- " << endl;
  fhGainNearVsGasDensity_profX->Sumw2();
  fhGainNearVsGasDensity_profX->Fit("lineGainNear","RM");

  double    GNP0    = lineGainNear->GetParameter(0);
  TString  sGNP0    = GetNumber(GNP0, 4);
  double errGNP0    = lineGainNear->GetParError(0);
  TString serrGNP0  = GetNumber(errGNP0, 1);
  double    GNP1    = lineGainNear->GetParameter(1);
  TString  sGNP1    = GetNumber(GNP1, 4);
  double errGNP1    = lineGainNear->GetParError(1);
  TString serrGNP1  = GetNumber(errGNP1, 1);

  TString stotGNP0 = "P_{0,N} = " + sGNP0 + " #pm " + serrGNP0;
  TString stotGNP1 = "P_{1,N} = " + sGNP1 + " #pm " + serrGNP1;

  TH2D *fhGainFarVsGasDensity = (TH2D*)f->Get("fhGainFarVsGasDensity");
  fhGainFarVsGasDensity->SetFillColor(0);
  fhGainFarVsGasDensity->GetXaxis()->SetRangeUser(0.985, 1.03);
  fhGainFarVsGasDensity->GetXaxis()->SetNdivisions(505);
  fhGainFarVsGasDensity->GetYaxis()->SetRangeUser(2000.0, 4000.0);
   // get profile
  fhGainFarVsGasDensity->ProfileX("fhGainFarVsGasDensity_profX");
  // fit
  cout << "---------------> Gain Far <------------------------- " << endl;
  fhGainFarVsGasDensity_profX->Sumw2();
  fhGainFarVsGasDensity_profX->Fit("lineGainFar","RM");

  double    GFP0    = lineGainFar->GetParameter(0);
  TString  sGFP0    = GetNumber(GFP0, 4);
  double errGFP0    = lineGainFar->GetParError(0);
  TString serrGFP0  = GetNumber(errGFP0, 1);
  double    GFP1    = lineGainFar->GetParameter(1);
  TString  sGFP1    = GetNumber(GFP1, 4);
  double errGFP1    = lineGainFar->GetParError(1);
  TString serrGFP1  = GetNumber(errGFP1, 1);

  TString stotGFP0 = "P_{0,F} = " + sGFP0 + " #pm " + serrGFP0;
  TString stotGFP1 = "P_{1,F} = " + sGFP1 + " #pm " + serrGFP1;

  // draw
  fhGainNearVsGasDensity->Draw("col1z");
  fhGainNearVsGasDensity_profX->Draw("same");

  fhGainFarVsGasDensity->Draw("sameCONT1");
  fhGainFarVsGasDensity_profX->Draw("same");

  TLegend *legG = new TLegend(0.42, 0.79, 0.897, 0.898);
  legG->SetFillColor(0);
  legG->SetBorderSize(0);
  legG->SetTextSize(0.04);
  legG->AddEntry(lineGainNear, "gas gain_{Near} = P_{0,N} + P_{1,N}#upoint#bar{#rho}","lp");
  legG->AddEntry(lineGainFar, "gas gain_{Far} = P_{0,F} + P_{1,F}#upoint#bar{#rho}","lp");
  legG->Draw();

  TLatex *infoG = new TLatex();
  infoG->SetTextSize(0.03);
  infoG->DrawLatex(1.015, 5100, stotGNP0.Data());
  infoG->DrawLatex(1.015, 4850, stotGNP1.Data());
  infoG->DrawLatex(1.015, 4600, stotGFP0.Data());
  infoG->DrawLatex(1.015, 4350, stotGFP1.Data());

}

TString GetNumber(double _number, int decimal)
{
  TString s; s += _number;
  int _exp = s.Index("e");
  int _dot = s.Index(".");
  if(_exp>0) {
    TString sExp = s;
    sExp.Remove(0,_exp);
  }
  s.Remove(_dot+decimal+1);
  if(_exp>0) {
    s.Append(sExp);
  }
  return s;
}
